@@ -25,6 +25,13 @@ from libopensesame.item_stack import item_stack_singleton
 from libopensesame.py3compat import *
 
 
+# A list of names that cannot be used, because they cause trouble for one reason
+# or another
+INVALID_NAMES = [
+	'response' # causes conflict with correct_response variable
+]
+
+
 class item_store(object):
 
 	"""
@@ -32,6 +39,8 @@ class item_store(object):
 		The `items` object provides dict-like access to the items. It's mainly
 		useful for programatically executing items.
 
+		An `items` object is created automatically when the experiment starts.
+
 		In addition to the functions listed below, the following semantics are
 		supported:
 
@@ -85,6 +94,9 @@ class item_store(object):
 			name:
 				desc:	An item name.
 				type:	str
+
+		example: |
+			items.execute(u'target_sketchpad')
 		"""
 
 		self.prepare(name)
@@ -100,6 +112,10 @@ class item_store(object):
 			name:
 				desc:	An item name.
 				type:	str
+
+		example: |
+			items.prepare('target_sketchpad')
+			items.run('target_sketchpad')
 		"""
 
 		item_stack_singleton.push(name, u'run')
@@ -116,6 +132,10 @@ class item_store(object):
 			name:
 				desc:	An item name.
 				type:	str
+
+		example: |
+			items.prepare('target_sketchpad')
+			items.run('target_sketchpad')
 		"""
 
 		item_stack_singleton.push(name, u'prepare')
@@ -196,6 +216,9 @@ class item_store(object):
 		returns:
 			desc:	A unique name.
 			type:	unicode
+
+		example: |
+			valid_name = items.valid_name(u'sketchpad', u'an invalid name')
 		"""
 
 		if suggestion is None:
@@ -211,14 +234,32 @@ class item_store(object):
 			elif name[0].isnumeric():
 				name = u'_' + name
 		_name = name
+		invalid_names = list(self) + INVALID_NAMES
 		i = 1
-		while _name in self:
+		while _name in invalid_names:
 			_name = u'%s_%d' % (name, i)
 			i += 1
 		return cistr(_name)
 
 	def _type(self, name):
 
+		"""
+		desc:
+			Gets the type of an item.
+
+		arguments:
+			name:
+				desc:	The name of an item.
+				type:	unicode
+
+		returns:
+			desc:	The type of an item, or `None` if the item doesn't exist.
+			type:	[unicode, NoneType]
+
+		example: |
+			print(items._type('target_sketchpad'))
+		"""
+
 		if name not in self:
 			return None
 		return self[name].item_type
@@ -243,38 +284,14 @@ class item_store(object):
 
 	@property
 	def items(self):
-
-		"""
-		name: items
-
-		desc:
-			Returns a list of (item name, item object) tuples.
-		"""
-
 		return self.__items__.items
 
 	@property
 	def keys(self):
-
-		"""
-		name: keys
-
-		desc:
-			Returns a list of item names.
-		"""
-
 		return self.__items__.keys
 
 	@property
 	def values(self):
-
-		"""
-		name: values
-
-		desc:
-			Returns a list of `item` objects.
-		"""
-
 		return self.__items__.values
 
 	@property
