@@ -17,7 +17,9 @@ You should have received a copy of the GNU General Public License
 along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+import functools
 import sys
+import io
 
 if sys.version_info >= (3,0,0):
 	py3 = True
@@ -28,6 +30,8 @@ else:
 	str = unicode
 	py3 = False
 	universal_newline_mode = u'rU'
+	RecursionError = RuntimeError
+
 
 def safe_decode(s, enc='utf-8', errors='strict'):
 	if isinstance(s, str):
@@ -51,19 +55,27 @@ def safe_decode(s, enc='utf-8', errors='strict'):
 	# encoding. This mostly applies to non-stringy things, such as integers.
 	return str(s)
 
+
 def safe_encode(s, enc='utf-8', errors='strict'):
 	if isinstance(s, bytes):
 		return s
 	return s.encode(enc, errors)
 
+
+def safe_read(path):
+	with open(path, u'r') as fd:
+		return safe_decode(fd.read(), errors=u'ignore')
+
+safe_open = functools.partial(io.open, encoding=u'utf-8')
+
 if py3:
 	safe_str = safe_decode
 else:
 	safe_str = safe_encode
 
 __all__ = ['py3', 'safe_decode', 'safe_encode', 'safe_str',
-	'universal_newline_mode']
+	'universal_newline_mode', 'safe_read', 'safe_open']
 if not py3:
-	__all__ += ['str', 'bytes']
+	__all__ += ['str', 'bytes', 'RecursionError']
 else:
 	__all__ += ['basestring']
