@@ -45,6 +45,7 @@ class general_properties(base_widget):
 
 		super(general_properties, self).__init__(main_window,
 			ui=u'widgets.general_properties')
+		self.lock = False
 		# Set the header, with the icon, label and script button
 		self.header_widget = general_header_widget(self, self.main_window)
 		header_hbox = QtWidgets.QHBoxLayout()
@@ -68,17 +69,18 @@ class general_properties(base_widget):
 				icon), self.backend_format % (name, desc))
 		self.quick_connect(
 			slot=self.main_window.ui.tabwidget.open_general_script,
-			signals=[
-				self.ui.button_script_editor.clicked,
-				self.ui.button_backend_settings.clicked
-			])
+			signals=[self.ui.button_script_editor.clicked],
+		)
+		self.quick_connect(
+			slot=self.main_window.ui.tabwidget.open_backend_settings,
+			signals=[self.ui.button_backend_settings.clicked]
+		)
 		self.quick_connect(
 			slot=self.apply_changes,
 			signals=[
 				self.ui.combobox_backend.currentIndexChanged,
 				self.ui.spinbox_width.editingFinished,
 				self.ui.spinbox_height.editingFinished,
-				self.ui.checkbox_bidi.stateChanged,
 				self.ui.checkbox_disable_garbage_collection.stateChanged,
 				self.ui.checkbox_uniform_coordinates.stateChanged,
 				self.ui.edit_foreground.textEdited,
@@ -176,8 +178,6 @@ class general_properties(base_widget):
 			self.ui.checkbox_disable_garbage_collection.isChecked()
 		self.experiment.var.uniform_coordinates = \
 			self.ui.checkbox_uniform_coordinates.isChecked()
-		self.experiment.var.bidi = self.ui.checkbox_bidi.isChecked()
-		self.check_bidi()
 		# Refresh the interface and unlock the general tab
 		self.lock = False
 		self.main_window.extension_manager.fire(u'change_experiment')
@@ -218,31 +218,9 @@ class general_properties(base_widget):
 		self.ui.edit_background.setText(safe_decode(
 			self.experiment.var.background))
 		self.ui.widget_font.initialize(self.experiment)
-		self.ui.checkbox_bidi.setChecked(self.experiment.var.bidi == u'yes')
 		self.ui.checkbox_disable_garbage_collection.setChecked(
 			self.experiment.var.disable_garbage_collection == u'yes')
 		self.ui.checkbox_uniform_coordinates.setChecked(
 			self.experiment.var.uniform_coordinates == u'yes')
-		self.check_bidi()
 		# Release the general tab
 		self.lock = False
-
-	def check_bidi(self):
-
-		"""
-		desc:
-			Shows the bidi-check message if bi-directional text support is
-			enabled while python-bidi is not installed.
-		"""
-
-		if self.experiment.var.bidi != u'yes':
-			self.ui.label_bidi_check.hide()
-			return
-		try:
-			import bidi
-		except:
-			bidi = None
-		if bidi is None:
-			self.ui.label_bidi_check.show()
-		else:
-			self.ui.label_bidi_check.hide()
